import * as _trpc_client from '@trpc/client';
import { TRPCRequestOptions, TRPCClient, CreateTRPCClientOptions, TRPCClientErrorLike } from '@trpc/client';
import * as _trpc_server from '@trpc/server';
import { AnyRouter, AnyProcedure, AnyQueryProcedure, inferProcedureInput, AnyMutationProcedure, AnySubscriptionProcedure, ProcedureRouterRecord, ProtectedIntersection } from '@trpc/server';
import { CreateQueryOptions, FetchQueryOptions, FetchInfiniteQueryOptions, InitialDataFunction, QueryOptions, CreateInfiniteQueryOptions, CreateMutationOptions, DehydratedState, CreateQueryResult, DefinedCreateQueryResult, QueryObserverSuccessResult, CreateInfiniteQueryResult, InfiniteQueryObserverSuccessResult, CreateMutationResult, InfiniteData, QueryClient } from '@tanstack/svelte-query';
import { inferTransformedProcedureOutput, inferTransformedSubscriptionOutput } from '@trpc/server/shared';
import { Readable } from 'svelte/store';

type OutputWithCursor<TData, TCursor = any> = {
    cursor: TCursor | null;
    data: TData;
};
interface TRPCSvelteRequestOptions extends Omit<TRPCRequestOptions, 'signal'> {
    /**
     * Opt out or into aborting request on unmount
     */
    abortOnUnmount?: boolean;
}
interface TRPCCreateQueryBaseOptions {
    /**
     * tRPC-related options
     */
    trpc?: TRPCSvelteRequestOptions;
}
interface CreateTRPCQueryOptions<TPath, TInput, TOutput, TData, TError> extends CreateQueryOptions<TOutput, TError, TData, [TPath, TInput]>, TRPCCreateQueryBaseOptions {
}
interface PrefetchTRPCQueryOptions<TPath, TInput, TOutput, TData, TError> extends FetchQueryOptions<TOutput, TError, TData, [TPath, TInput]> {
}
interface PrefetchTRPCInfiniteQueryOptions<TPath, TInput, TOutput, TData, TError> extends FetchInfiniteQueryOptions<TOutput, TError, TData, [TPath, TInput]> {
}
/** @internal **/
interface DefinedUseTRPCQueryOptions<TPath, TInput, TOutput, TData, TError> extends CreateTRPCQueryOptions<TPath, TInput, TOutput, TData, TError> {
    initialData: TOutput | InitialDataFunction<TOutput>;
}
interface TRPCQueryOptions<TPath, TInput, TData, TError> extends QueryOptions<TData, TError, TData, [TPath, TInput]>, TRPCCreateQueryBaseOptions {
}
type ExtractCursorType<TInput> = TInput extends {
    cursor: any;
} ? TInput['cursor'] : unknown;
interface CreateTRPCInfiniteQueryOptions<TPath, TInput, TOutput, TError> extends CreateInfiniteQueryOptions<TOutput, TError, TOutput, TOutput, [
    TPath,
    Omit<TInput, 'cursor'>
]>, TRPCCreateQueryBaseOptions {
    initialCursor?: ExtractCursorType<TInput>;
}
interface CreateTRPCMutationOptions<TInput, TError, TOutput, TContext = unknown> extends CreateMutationOptions<TOutput, TError, TInput, TContext>, TRPCCreateQueryBaseOptions {
}
interface CreateTRPCSubscriptionOptions<TOutput, TError> {
    enabled?: boolean;
    onStarted?: () => void;
    onData: (data: TOutput) => void;
    onError?: (err: TError) => void;
}
type GetDehydratedState<TRouter extends AnyRouter> = (client: TRPCClient<TRouter>, trpcState: DehydratedState | undefined) => DehydratedState | undefined;
type CreateClient<TRouter extends AnyRouter> = (opts: CreateTRPCClientOptions<TRouter>) => TRPCClient<TRouter>;
/**
 * @internal
 */
type CreateTRPCQueryResult<TData, TError> = CreateQueryResult<TData, TError>;
/**
 * @internal
 */
type DefinedCreateTRPCQueryResult<TData, TError> = DefinedCreateQueryResult<TData, TError>;
/**
 * @internal
 */
type UseTRPCQuerySuccessResult<TData, TError> = QueryObserverSuccessResult<TData, TError>;
/**
 * @internal
 */
type CreateTRPCInfiniteQueryResult<TData, TError> = CreateInfiniteQueryResult<TData, TError>;
/**
 * @internal
 */
type UseTRPCInfiniteQuerySuccessResult<TData, TError> = InfiniteQueryObserverSuccessResult<TData, TError>;
/**
 * @internal
 */
type CreateTRPCMutationResult<TData, TError, TVariables, TContext> = CreateMutationResult<TData, TError, TVariables, TContext>;

/**
 * @internal
 */
interface ProcedureCreateQuery<TProcedure extends AnyProcedure, TPath extends string> {
    <TQueryFnData = inferTransformedProcedureOutput<TProcedure>, TData = inferTransformedProcedureOutput<TProcedure>>(input: inferProcedureInput<TProcedure>, opts: DefinedUseTRPCQueryOptions<TPath, inferProcedureInput<TProcedure>, TQueryFnData, TData, TRPCClientErrorLike<TProcedure>>): DefinedCreateTRPCQueryResult<TData, TRPCClientErrorLike<TProcedure>>;
    <TQueryFnData = inferTransformedProcedureOutput<TProcedure>, TData = inferTransformedProcedureOutput<TProcedure>>(input: inferProcedureInput<TProcedure>, opts?: CreateTRPCQueryOptions<TPath, inferProcedureInput<TProcedure>, TQueryFnData, TData, TRPCClientErrorLike<TProcedure>>): CreateTRPCQueryResult<TData, TRPCClientErrorLike<TProcedure>>;
}
interface ProcedurePrefetchQuery<TProcedure extends AnyProcedure, TPath extends string> {
    <TQueryFnData = inferTransformedProcedureOutput<TProcedure>, TData = inferTransformedProcedureOutput<TProcedure>>(input: inferProcedureInput<TProcedure>, opts?: PrefetchTRPCQueryOptions<TPath, inferProcedureInput<TProcedure>, TQueryFnData, TData, TRPCClientErrorLike<TProcedure>>): Promise<void>;
}
/**
 * @internal
 */
type DecorateProcedure<TProcedure extends AnyProcedure, TFlags, TPath extends string> = TProcedure extends AnyQueryProcedure ? {
    query: ProcedureCreateQuery<TProcedure, TPath>;
    prefetchQuery: ProcedurePrefetchQuery<TProcedure, TPath>;
} & (inferProcedureInput<TProcedure> extends {
    cursor?: any;
} ? {
    useInfiniteQuery: <_TQueryFnData = inferTransformedProcedureOutput<TProcedure>, TData = inferTransformedProcedureOutput<TProcedure>>(input: Omit<inferProcedureInput<TProcedure>, 'cursor'>, opts?: CreateTRPCInfiniteQueryOptions<TPath, inferProcedureInput<TProcedure>, TData, TRPCClientErrorLike<TProcedure>>) => CreateTRPCInfiniteQueryResult<TData, TRPCClientErrorLike<TProcedure>>;
    prefetchInfiniteQuery(): Promise<void>;
} & (TFlags extends 'ExperimentalSuspense' ? {
    useSuspenseInfiniteQuery: <_TQueryFnData = inferTransformedProcedureOutput<TProcedure>, TData = inferTransformedProcedureOutput<TProcedure>>(input: Omit<inferProcedureInput<TProcedure>, 'cursor'>, opts?: Omit<CreateTRPCInfiniteQueryOptions<TPath, inferProcedureInput<TProcedure>, TData, TRPCClientErrorLike<TProcedure>>, 'enabled' | 'suspense'>) => [
        InfiniteData<TData>,
        UseTRPCInfiniteQuerySuccessResult<TData, TRPCClientErrorLike<TProcedure>>
    ];
} : object) : object) & (TFlags extends 'ExperimentalSuspense' ? {
    useSuspenseQuery: <TQueryFnData = inferTransformedProcedureOutput<TProcedure>, TData = inferTransformedProcedureOutput<TProcedure>>(input: inferProcedureInput<TProcedure>, opts?: Omit<CreateTRPCQueryOptions<TPath, inferProcedureInput<TProcedure>, TQueryFnData, TData, TRPCClientErrorLike<TProcedure>>, 'enabled' | 'suspense'>) => [
        TData,
        UseTRPCQuerySuccessResult<TData, TRPCClientErrorLike<TProcedure>>
    ];
} : object) : TProcedure extends AnyMutationProcedure ? {
    mutation: <TContext = unknown>(opts?: CreateTRPCMutationOptions<inferProcedureInput<TProcedure>, TRPCClientErrorLike<TProcedure>, inferTransformedProcedureOutput<TProcedure>, TContext>) => CreateTRPCMutationResult<inferTransformedProcedureOutput<TProcedure>, TRPCClientErrorLike<TProcedure>, inferProcedureInput<TProcedure>, TContext>;
} : TProcedure extends AnySubscriptionProcedure ? {
    subscription: (input: inferProcedureInput<TProcedure>, opts?: CreateTRPCSubscriptionOptions<inferTransformedSubscriptionOutput<TProcedure>, TRPCClientErrorLike<TProcedure>>) => {
        state: Readable<'idle' | 'running' | 'stopped'>;
        data: Readable<inferTransformedSubscriptionOutput<TProcedure> | undefined>;
        error: Readable<TRPCClientErrorLike<TProcedure> | undefined>;
        unsubscribe: () => void;
    };
} : never;
/**
 * @internal
 */
type DecoratedProcedureRecord<TProcedures extends ProcedureRouterRecord, TFlags, TPath extends string = ''> = {
    [TKey in keyof TProcedures]: TProcedures[TKey] extends AnyRouter ? DecoratedProcedureRecord<TProcedures[TKey]['_def']['record'], TFlags, `${TPath}${TKey & string}.`> : TProcedures[TKey] extends AnyProcedure ? DecorateProcedure<TProcedures[TKey], TFlags, `${TPath}${TKey & string}`> : never;
};
/**
 * @internal
 */
type CreateTRPCSvelteBase = {
    queryClient: QueryClient;
};
type CreateTRPCSvelte<TRouter extends AnyRouter, TFlags> = ProtectedIntersection<CreateTRPCSvelteBase, DecoratedProcedureRecord<TRouter['_def']['record'], TFlags>>;
declare function createTRPCSvelteClient<TRouter extends AnyRouter, TFlags = null>(opts: CreateTRPCClientOptions<TRouter> & {
    queryClient: QueryClient;
    fetch: typeof fetch;
}): CreateTRPCSvelte<TRouter, TFlags>;
declare function createTRPCSvelte<TRouter extends AnyRouter, TFlags = null>(): {
    createClient: (opts: (TRouter["_def"]["_config"]["transformer"] extends _trpc_server.DefaultDataTransformer ? {
        transformer?: "You must set a transformer on the backend router" | undefined;
    } : TRouter["_def"]["_config"]["transformer"] extends _trpc_server.DataTransformerOptions ? {
        transformer: TRouter["_def"]["_config"]["transformer"] extends _trpc_server.CombinedDataTransformer ? _trpc_server.DataTransformerOptions : TRouter["_def"]["_config"]["transformer"];
    } : {
        transformer?: _trpc_server.ClientDataTransformerOptions | undefined;
    }) & {
        links: _trpc_client.TRPCLink<TRouter>[];
    } & {
        queryClient: QueryClient;
        fetch: typeof fetch;
    }) => ProtectedIntersection<CreateTRPCSvelteBase, DecoratedProcedureRecord<TRouter["_def"]["record"], TFlags, "">>;
    setContextClient: (client: ProtectedIntersection<CreateTRPCSvelteBase, DecoratedProcedureRecord<TRouter["_def"]["record"], TFlags, "">>) => void;
    getContextClient: () => ProtectedIntersection<CreateTRPCSvelteBase, DecoratedProcedureRecord<TRouter["_def"]["record"], TFlags, "">>;
};

export { CreateTRPCSvelte as C, DecoratedProcedureRecord as D, ExtractCursorType as E, GetDehydratedState as G, OutputWithCursor as O, PrefetchTRPCQueryOptions as P, TRPCSvelteRequestOptions as T, UseTRPCQuerySuccessResult as U, createTRPCSvelte as a, CreateTRPCSvelteBase as b, createTRPCSvelteClient as c, DecorateProcedure as d, TRPCCreateQueryBaseOptions as e, CreateTRPCQueryOptions as f, PrefetchTRPCInfiniteQueryOptions as g, DefinedUseTRPCQueryOptions as h, TRPCQueryOptions as i, CreateTRPCInfiniteQueryOptions as j, CreateTRPCMutationOptions as k, CreateTRPCSubscriptionOptions as l, CreateClient as m, CreateTRPCQueryResult as n, DefinedCreateTRPCQueryResult as o, CreateTRPCInfiniteQueryResult as p, UseTRPCInfiniteQuerySuccessResult as q, CreateTRPCMutationResult as r };
