"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __reExport = (target, mod, secondTarget) => (__copyProps(target, mod, "default"), secondTarget && __copyProps(secondTarget, mod, "default"));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/index.ts
var src_exports = {};
__export(src_exports, {
  createTRPCSvelte: () => createTRPCSvelte,
  createTRPCSvelteClient: () => createTRPCSvelteClient
});
module.exports = __toCommonJS(src_exports);
__reExport(src_exports, require("@trpc/client"), module.exports);

// src/createTRPCSvelte.ts
var import_svelte_query2 = require("@tanstack/svelte-query");
var import_client = require("@trpc/client");
var import_shared = require("@trpc/server/shared");
var import_esm_env2 = require("esm-env");
var import_store = require("svelte/store");

// src/internals/getArrayQueryKey.ts
function getArrayQueryKey(path, input, type) {
  const hasInput = typeof input !== "undefined";
  const hasType = type && type !== "any";
  if (!hasInput && !hasType)
    return path.length ? [path] : [];
  const inputAndType = {};
  if (hasInput)
    inputAndType.input = input;
  if (hasType)
    inputAndType.type = type;
  return [path, inputAndType];
}

// src/internals/context.ts
var import_svelte = require("svelte");
var trpcContextKey = Symbol("trpcContext");
function setContextClient(client) {
  (0, import_svelte.setContext)(trpcContextKey, client);
}
function getContextClient() {
  return (0, import_svelte.getContext)(trpcContextKey);
}

// src/shared/utils.ts
var import_svelte_query = require("@tanstack/svelte-query");
var import_esm_env = require("esm-env");

// src/utils/splitUserOptions.ts
function splitUserOptions(options) {
  if (options === void 0) {
    return [];
  }
  const trpcOptions = {
    context: options.context,
    signal: options.signal
  };
  delete options.context;
  delete options.signal;
  return [trpcOptions, options];
}

// src/shared/utils.ts
var queryTypes = {
  invalidate: "any",
  prefetch: "query",
  prefetchInfinite: "infinite",
  fetch: "query",
  fetchInfinite: "infinite",
  refetch: "any",
  cancel: "any",
  reset: "any",
  setData: "query",
  setInfiniteData: "infinite",
  getData: "query",
  getInfiniteData: "infinite"
};
function callUtilMethod(trpc, client, path, method, args) {
  if (!import_esm_env.BROWSER) {
    client = (0, import_svelte_query.useQueryClient)();
  }
  const queryType = queryTypes[method];
  const queryKey = getArrayQueryKey(path, args[0], queryType);
  switch (method) {
    case "prefetch":
    case "fetch":
    case "prefetchInfinite":
    case "fetchInfinite": {
      const joinedPath = path.join(".");
      const options = args[1];
      const [trpcOptions, tanstackQueryOptions] = splitUserOptions(options);
      const queryFn = queryType === "query" ? () => trpc.query(joinedPath, args[0], trpcOptions) : (context) => {
        const input = { ...args[0], cursor: context.pageParam };
        return trpc.query(joinedPath, input, trpcOptions);
      };
      return client[method + "Query"]({
        ...tanstackQueryOptions,
        queryKey,
        queryFn
      });
    }
    case "invalidate":
      return client.invalidateQueries(queryKey, args[1], args[2]);
    case "refetch":
      return client.refetchQueries(
        getArrayQueryKey(path, args[0], "query"),
        args[1],
        args[2]
      );
    case "cancel":
      return client.cancelQueries(queryKey, args[1], args[2]);
    case "reset":
      return client.resetQueries(queryKey, args[1], args[2]);
    case "setData":
    case "setInfiniteData":
      return client.setQueryData(queryKey, args[1], args[2]);
    case "getData":
    case "getInfiniteData":
      return client.getQueryData(queryKey, args[1]);
    default:
      throw new TypeError(`trpc.${path}.${method} is not a function`);
  }
}

// src/createTRPCSvelte.ts
var clientMethods = {
  query: [1, "query"],
  mutation: [0, "any"],
  infiniteQuery: [1, "infinite"],
  prefetchQuery: [1, "query"],
  prefetchInfiniteQuery: [1, "infinite"],
  subscription: [1, "query"]
};
function createHooksInternalProxy(client, opts) {
  const queryClient = opts.queryClient;
  return (0, import_shared.createFlatProxy)((firstPath) => {
    switch (firstPath) {
      case "queryClient": {
        return queryClient;
      }
    }
    return (0, import_shared.createRecursiveProxy)(({ path, args: unknownArgs }) => {
      path.unshift(firstPath);
      const method = path.pop();
      const joinedPath = path.join(".");
      const methodData = clientMethods[method];
      if (!methodData) {
        const utils = path.pop();
        if (utils === "utils") {
          return callUtilMethod(
            client,
            queryClient,
            path,
            method,
            unknownArgs
          );
        }
        throw new TypeError(`trpc.${joinedPath}.${method} is not a function`);
      }
      const args = unknownArgs;
      const [optionIndex, queryType] = methodData;
      const options = args[optionIndex];
      const [trpcOptions, tanstackQueryOptions] = splitUserOptions(options);
      const key = getArrayQueryKey(
        path,
        method === "mutation" ? void 0 : args[0],
        queryType
      );
      const enabled = tanstackQueryOptions?.enabled !== false;
      switch (method) {
        case "query":
          return (0, import_svelte_query2.createQuery)({
            ...tanstackQueryOptions,
            enabled,
            queryKey: key,
            queryFn: () => client.query(joinedPath, args[0], trpcOptions)
          });
        case "mutation": {
          return (0, import_svelte_query2.createMutation)({
            ...tanstackQueryOptions,
            mutationKey: key,
            mutationFn: (variables) => client.mutation(joinedPath, variables, trpcOptions)
          });
        }
        case "subscription": {
          const state = (0, import_store.writable)("idle");
          const data = (0, import_store.writable)(void 0);
          const error = (0, import_store.writable)(
            void 0
          );
          const enabled2 = (options?.enabled ?? true) && import_esm_env2.BROWSER;
          if (!enabled2) {
            return {
              state,
              data,
              error,
              unsubscribe: () => void 0
            };
          }
          const subscription = client.subscription(joinedPath, args[0], {
            onStarted: () => {
              state.set("running");
            },
            onData: (data_) => {
              data.set(data_);
            },
            onError: (error_) => {
              error.set(error_);
            }
          });
          const unsubscribe = () => {
            state.set("stopped");
            subscription.unsubscribe();
          };
          return {
            state,
            data,
            error,
            unsubscribe
          };
        }
        case "infiniteQuery":
          return (0, import_svelte_query2.createInfiniteQuery)({
            ...tanstackQueryOptions,
            enabled,
            queryKey: key,
            queryFn: (context) => {
              const input = { ...args[0], cursor: context.pageParam };
              return client.query(joinedPath, input, trpcOptions);
            }
          });
        case "prefetchQuery": {
          return queryClient.prefetchQuery({
            ...tanstackQueryOptions,
            queryKey: key,
            queryFn: () => client.query(joinedPath, args[0], trpcOptions)
          });
        }
        case "prefetchInfiniteQuery": {
          return queryClient.prefetchInfiniteQuery({
            ...tanstackQueryOptions,
            queryKey: key,
            queryFn: (context) => {
              const input = { ...args[0], cursor: context.pageParam };
              return client.query(joinedPath, input, trpcOptions);
            }
          });
        }
        default:
          throw new TypeError(`trpc.${joinedPath}.${method} is not a function`);
      }
    });
  });
}
function createTRPCSvelteClient(opts) {
  const client = (0, import_client.createTRPCUntypedClient)(opts);
  const proxy = createHooksInternalProxy(client, opts);
  return proxy;
}
function createTRPCSvelte() {
  const createClient = createTRPCSvelteClient;
  const setContextClient2 = setContextClient;
  const getContextClient2 = getContextClient;
  return {
    createClient,
    setContextClient: setContextClient2,
    getContextClient: getContextClient2
  };
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  createTRPCSvelte,
  createTRPCSvelteClient
});
